@@ -3,12 +3,12 @@
    Lexical scanner for dhcpd config file... */
 
 /*
- * Copyright (c) 2004-2011 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -24,12 +24,6 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -153,13 +147,21 @@ save_parse_state(struct parse *cfile) {
 /*
  * Return the parser to the previous saved state.
  *
- * You must call save_parse_state() before calling 
- * restore_parse_state(), but you can call restore_parse_state() any
- * number of times after that.
+ * You must call save_parse_state() every time before calling
+ * restore_parse_state().
+ *
+ * Note: When the read function callback is in use in ldap mode,
+ * a call to get_char() may reallocate the buffer and will append
+ * config data to the buffer until a state restore.
+ * Do not restore to the (freed) pointer and size, but use new one.
  */
 isc_result_t
 restore_parse_state(struct parse *cfile) {
 	struct parse *saved_state;
+#if defined(LDAP_CONFIGURATION)
+	char *inbuf = cfile->inbuf;
+	size_t size = cfile->bufsiz;
+#endif
 
 	if (cfile->saved_state == NULL) {
 		return DHCP_R_NOTYET;
@@ -167,7 +169,13 @@ restore_parse_state(struct parse *cfile) {
 
 	saved_state = cfile->saved_state;
 	memcpy(cfile, saved_state, sizeof(*cfile));
-	cfile->saved_state = saved_state;
+	dfree(saved_state, MDL);
+	cfile->saved_state = NULL;
+
+#if defined(LDAP_CONFIGURATION)
+	cfile->inbuf = inbuf;
+	cfile->bufsiz = size;
+#endif
 	return ISC_R_SUCCESS;
 }
 
@@ -470,8 +478,20 @@ read_whitespace(int c, struct parse *cfile) {
 	 */
 	ofs = 0;
 	do {
+		if (ofs >= (sizeof(cfile->tokbuf) - 1)) {
+			/*
+			 * As the file includes a huge amount of whitespace,
+			 * it's probably broken.
+			 * Print out a warning and bail out.
+			 */
+			parse_warn(cfile,
+				   "whitespace too long, buffer overflow.");
+			log_fatal("Exiting");
+		}
 		cfile->tokbuf[ofs++] = c;
 		c = get_char(cfile);
+		if (c == EOF)
+			return END_OF_FILE;
 	} while (!((c == '\n') && cfile->eol_token) && 
 		 isascii(c) && isspace(c));
 
@@ -534,7 +554,6 @@ static enum dhcp_token read_string (cfile)
 				goto again;
 			      default:
 				cfile -> tokbuf [i] = c;
-				bs = 0;
 				break;
 			}
 			bs = 0;
@@ -764,6 +783,8 @@ intern(char *atom, enum dhcp_token dfv) {
 				return ATSFP;
 			break;
 		}
+		if (!strcasecmp(atom + 1, "uthoring-byte-order"))
+			return AUTHORING_BYTE_ORDER;
 		if (!strncasecmp(atom + 1, "ut", 2)) {
 			if (isascii(atom[3]) &&
 			    (tolower((unsigned char)atom[3]) == 'h')) {
@@ -808,6 +829,9 @@ intern(char *atom, enum dhcp_token dfv) {
 			return BALANCE;
 		if (!strcasecmp (atom + 1, "ound"))
 			return BOUND;
+		if (!strcasecmp(atom+1, "ig-endian")) {
+			return TOKEN_BIG_ENDIAN;
+		}
 		break;
 	      case 'c':
 		if (!strcasecmp(atom + 1, "ase"))
@@ -877,10 +901,6 @@ intern(char *atom, enum dhcp_token dfv) {
 	      case 'd':
 		if (!strcasecmp(atom + 1, "b-time-format"))
 			return DB_TIME_FORMAT;
-		if (!strcasecmp (atom + 1, "ns-update"))
-			return DNS_UPDATE;
-		if (!strcasecmp (atom + 1, "ns-delete"))
-			return DNS_DELETE;
 		if (!strcasecmp (atom + 1, "omain"))
 			return DOMAIN;
 		if (!strncasecmp (atom + 1, "omain-", 6)) {
@@ -889,6 +909,9 @@ intern(char *atom, enum dhcp_token dfv) {
 			if (!strcasecmp(atom + 7, "list"))
 				return DOMAIN_LIST;
 		}
+		if (!strcasecmp (atom + 1, "o-forward-updates"))
+			return DO_FORWARD_UPDATE;
+		/* do-forward-update is included for historical reasons */
 		if (!strcasecmp (atom + 1, "o-forward-update"))
 			return DO_FORWARD_UPDATE;
 		if (!strcasecmp (atom + 1, "ebug"))
@@ -1027,6 +1050,9 @@ intern(char *atom, enum dhcp_token dfv) {
 			return HOSTNAME;
 		if (!strcasecmp (atom + 1, "elp"))
 			return TOKEN_HELP;
+		if (!strcasecmp (atom + 1, "ex")) {
+			return TOKEN_HEX;
+		}
 		break;
 	      case 'i':
 	      	if (!strcasecmp(atom+1, "a-na")) 
@@ -1043,6 +1069,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return INCLUDE;
 		if (!strcasecmp (atom + 1, "nteger"))
 			return INTEGER;
+		if (!strcasecmp (atom  + 1, "nfiniband"))
+			return TOKEN_INFINIBAND;
 		if (!strcasecmp (atom + 1, "nfinite"))
 			return INFINITE;
 		if (!strcasecmp (atom + 1, "nfo"))
@@ -1053,8 +1081,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return IP6_ADDRESS;
 		if (!strcasecmp (atom + 1, "nitial-interval"))
 			return INITIAL_INTERVAL;
-                if (!strcasecmp (atom + 1, "nitial-delay"))
-                        return INITIAL_DELAY;
+		if (!strcasecmp (atom + 1, "nitial-delay"))
+			return INITIAL_DELAY;
 		if (!strcasecmp (atom + 1, "nterface"))
 			return INTERFACE;
 		if (!strcasecmp (atom + 1, "dentifier"))
@@ -1076,6 +1104,8 @@ intern(char *atom, enum dhcp_token dfv) {
 		}
 		if (!strcasecmp (atom + 1, "ey"))
 			return KEY;
+		if (!strcasecmp (atom + 1, "ey-algorithm"))
+			return KEY_ALGORITHM;
 		break;
 	      case 'l':
 		if (!strcasecmp (atom + 1, "case"))
@@ -1108,6 +1138,12 @@ intern(char *atom, enum dhcp_token dfv) {
 		if (!strcasecmp(atom+1, "l")) {
 			return LL;
 		}
+		if (!strcasecmp(atom+1, "ittle-endian")) {
+			return TOKEN_LITTLE_ENDIAN;
+		}
+		if (!strcasecmp (atom + 1, "ease-id-format")) {
+			return LEASE_ID_FORMAT;
+		}
 		break;
 	      case 'm':
 		if (!strncasecmp (atom + 1, "ax", 2)) {
@@ -1172,8 +1208,6 @@ intern(char *atom, enum dhcp_token dfv) {
 			return TOKEN_NOT;
 		if (!strcasecmp (atom + 1, "o"))
 			return TOKEN_NO;
-		if (!strcasecmp (atom + 1, "s-update"))
-			return NS_UPDATE;
 		if (!strcasecmp (atom + 1, "oerror"))
 			return NS_NOERROR;
 		if (!strcasecmp (atom + 1, "otauth"))
@@ -1210,8 +1244,13 @@ intern(char *atom, enum dhcp_token dfv) {
 			return OF;
 		if (!strcasecmp (atom + 1, "wner"))
 			return OWNER;
+		if (!strcasecmp (atom + 1, "ctal")) {
+			return TOKEN_OCTAL;
+		}
 		break;
 	      case 'p':
+		if (!strcasecmp (atom + 1, "arse-vendor-option"))
+			return PARSE_VENDOR_OPT;
 		if (!strcasecmp (atom + 1, "repend"))
 			return PREPEND;
 		if (!strcasecmp(atom + 1, "referred-life"))
@@ -1220,6 +1259,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return PACKET;
 		if (!strcasecmp (atom + 1, "ool"))
 			return POOL;
+		if (!strcasecmp (atom + 1, "ool6"))
+			return POOL6;
 		if (!strcasecmp (atom + 1, "refix6"))
 			return PREFIX6;
 		if (!strcasecmp (atom + 1, "seudo"))
@@ -1228,6 +1269,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return PEER;
 		if (!strcasecmp (atom + 1, "rimary"))
 			return PRIMARY;
+		if (!strcasecmp (atom + 1, "rimary6"))
+			return PRIMARY6;
 		if (!strncasecmp (atom + 1, "artner", 6)) {
 			if (!atom [7])
 				return PARTNER;
@@ -1317,7 +1360,9 @@ intern(char *atom, enum dhcp_token dfv) {
 			    tolower((unsigned char)atom[2]) == 'c') {
 				if (!strncasecmp(atom + 3, "ond", 3)) {
                                         if (!strcasecmp(atom + 6, "ary"))
-                                                return SECONDARY;
+						return SECONDARY;
+                                        if (!strcasecmp(atom + 6, "ary6"))
+						return SECONDARY6;
                                         if (!strcasecmp(atom + 6, "s"))
                                                 return SECONDS;
 					break;
@@ -1486,12 +1531,14 @@ intern(char *atom, enum dhcp_token dfv) {
 		}
 		if (!strcasecmp (atom + 1, "nauthenticated"))
 			return UNAUTHENTICATED;
-		if (!strcasecmp (atom + 1, "pdated-dns-rr"))
-			return UPDATED_DNS_RR;
 		if (!strcasecmp (atom + 1, "pdate"))
 			return UPDATE;
 		break;
 	      case 'v':
+		if (!strcasecmp (atom + 1, "6relay"))
+			return V6RELAY;
+		if (!strcasecmp (atom + 1, "6relopt"))
+			return V6RELOPT;
 		if (!strcasecmp (atom + 1, "endor-class"))
 			return VENDOR_CLASS;
 		if (!strcasecmp (atom + 1, "endor"))
