@@ -3,12 +3,12 @@
    Tables of information... */
 
 /*
- * Copyright (c) 2004-2016 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2018 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -191,9 +191,9 @@ static struct option dhcp_options[] = {
 
 	{ "client-last-transaction-time", "L",  &dhcp_universe,  91, 1 },
 	{ "associated-ip", "Ia",                &dhcp_universe,  92, 1 },
-#if 0
+#if defined(RFC4578_OPTIONS)
 	/* Defined by RFC 4578 */
-	{ "pxe-system-type", "S",		&dhcp_universe,  93, 1 },
+	{ "pxe-system-type", "Sa",		&dhcp_universe,  93, 1 },
 	{ "pxe-interface-id", "BBB",		&dhcp_universe,  94, 1 },
 	{ "pxe-client-id", "BX",		&dhcp_universe,  97, 1 },
 #endif
@@ -250,8 +250,7 @@ static struct option dhcp_options[] = {
 #if defined(RFC6731_OPTIONS)
         { "rdnss-selection", "BIID",		&dhcp_universe, 146, 1 },
 #endif
-#if 0
-	/* Not defined by RFC yet */
+#if defined(RFC5859_OPTIONS)
 	{ "tftp-server-address", "Ia",		&dhcp_universe, 150, 1 },
 #endif
 #if defined(RFC7618_OPTIONS)
@@ -260,9 +259,11 @@ static struct option dhcp_options[] = {
 #if defined(RFC7710_OPTIONS)
 	{ "v4-captive-portal", "t",		&dhcp_universe, 160, 1 },
 #endif
-#if 0
-	/* PXELINUX options: defined by RFC 5071 */
+#if defined(RFC5071_OPTIONS)
+#if  0
+	/* Option 208 has been officially deprecated. Do NOT define it */
 	{ "pxelinux-magic", "BBBB",		&dhcp_universe, 208, 1 },
+#endif
 	{ "loader-configfile", "t",		&dhcp_universe, 209, 1 },
 	{ "loader-pathprefix", "t",		&dhcp_universe, 210, 1 },
 	{ "loader-reboottime", "L",		&dhcp_universe, 211, 1 },
@@ -365,10 +366,10 @@ static struct option dhcpv6_options[] = {
 	{ "relay-msg", "X",			&dhcpv6_universe,  9, 1 },
 
 	/* Option code 10 is curiously unassigned. */
-	/* 
+	/*
 	 * In draft-ietf-dhc-dhcpv6-25 there were two OPTION_CLIENT_MSG and
 	 * OPTION_SERVER_MSG options. They were eventually unified as
-	 * OPTION_RELAY_MSG, hence no option with value of 10. 
+	 * OPTION_RELAY_MSG, hence no option with value of 10.
 	 */
 #if 0
 	/* XXX: missing suitable atoms for the auth option.  We may want
@@ -557,12 +558,6 @@ static struct option dhcpv6_options[] = {
 	{ "solmax-rt", "L",			&dhcpv6_universe, 82, 1 },
 	{ "inf-max-rt", "L",			&dhcpv6_universe, 83, 1 },
 #endif
-#if defined(RFC7710_OPTIONS)
-	{ "v6-captive-portal", "t",		&dhcpv6_universe, 103, 1 },
-#endif
-#if defined(RFC6153_OPTIONS)
-	{ "ipv6-address-andsf", "6A",		&dhcpv6_universe, 143, 1 },
-#endif
 
 			/* RFC7341 OPTIONS */
 #if defined(RFC7341_OPTIONS)
@@ -570,6 +565,16 @@ static struct option dhcpv6_options[] = {
 	{ "dhcp4-o-dhcp6-server", "6A",		&dhcpv6_universe, 88, 1 },
 #endif
 
+#if defined(RFC7710_OPTIONS)
+	{ "v6-captive-portal", "t",		&dhcpv6_universe, 103, 1 },
+#endif
+
+	{ "relay-source-port", "S",		&dhcpv6_universe, 135, 1 },
+
+#if defined(RFC6153_OPTIONS)
+	{ "ipv6-address-andsf", "6A",		&dhcpv6_universe, 143, 1 },
+#endif
+
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
