@@ -3,13 +3,12 @@
    Operating system dependencies... */
 
 /*
- * Copyright (c) 2004-2005,2007-2010,2014 by Internet Systems Consortium,
- *                                        Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium,Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -100,7 +99,7 @@
 #ifdef USE_SOCKETS
 #  define USE_SOCKET_SEND
 #  define USE_SOCKET_RECEIVE
-#  if defined(HAVE_DLPI)
+#  if defined(HAVE_DLPI) && !defined(sun) && !defined(USE_V4_PKTINFO)
 #    define USE_DLPI_HWADDR
 #  elif defined(HAVE_LPF)
 #    define USE_LPF_HWADDR
