@@ -3,12 +3,12 @@
    Persistent database management routines for DHCPD... */
 
 /*
- * Copyright (c) 2004-2016 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -106,7 +106,7 @@ int write_lease (lease)
 	/* If the lease file is corrupt, don't try to write any more leases
 	   until we've written a good lease file. */
 	if (lease_file_is_corrupt)
-		if (!new_lease_file ())
+		if (!new_lease_file (0))
 			return 0;
 
 	if (counting)
@@ -305,7 +305,7 @@ int write_host (host)
 	/* If the lease file is corrupt, don't try to write any more leases
 	   until we've written a good lease file. */
 	if (lease_file_is_corrupt)
-		if (!new_lease_file ())
+		if (!new_lease_file (0))
 			return 0;
 
 	if (!db_printable((unsigned char *)host->name))
@@ -374,7 +374,7 @@ int write_host (host)
 					++errors;
 			}
 		}
-		
+
 		memset (&ip_addrs, 0, sizeof ip_addrs);
 		if (host -> fixed_addr &&
 		    evaluate_option_cache (&ip_addrs, (struct packet *)0,
@@ -384,7 +384,7 @@ int write_host (host)
 					   (struct option_state *)0,
 					   &global_scope,
 					   host -> fixed_addr, MDL)) {
-		
+
 			errno = 0;
 			fprintf (db_file, "\n  fixed-address ");
 			if (errno)
@@ -454,7 +454,7 @@ int write_group (group)
 	/* If the lease file is corrupt, don't try to write any more leases
 	   until we've written a good lease file. */
 	if (lease_file_is_corrupt)
-		if (!new_lease_file ())
+		if (!new_lease_file (0))
 			return 0;
 
 	if (!db_printable((unsigned char *)group->name))
@@ -525,12 +525,33 @@ write_ia(const struct ia_xx *ia) {
 	char *s;
 	int fprintf_ret;
 
-	/* 
-	 * If the lease file is corrupt, don't try to write any more 
-	 * leases until we've written a good lease file. 
+#ifdef EUI_64
+	/* If we're not writing EUI64 leases to the file, then
+	* we can skip writing this IA provided all of its leases
+	* are EUI64. (Not sure you can ever have a case where
+	* they aren't but doesn't hurt to check) */
+	if (ia->ia_type == D6O_IA_NA && !persist_eui64) {
+		int i;
+		for (i=0; i < ia->num_iasubopt; i++) {
+			if (!ia->iasubopt[i]->ipv6_pool->ipv6_pond->use_eui_64)
+			{
+				break;
+			}
+		}
+
+		if (i == ia->num_iasubopt) {
+			/* Their all EUI64 so we can skip it */
+			return(1);
+		}
+	}
+#endif
+
+	/*
+	 * If the lease file is corrupt, don't try to write any more
+	 * leases until we've written a good lease file.
 	 */
 	if (lease_file_is_corrupt) {
-		if (!new_lease_file()) {
+		if (!new_lease_file(0)) {
 			return 0;
 		}
 	}
@@ -587,11 +608,11 @@ write_ia(const struct ia_xx *ia) {
 			goto error_exit;
 		}
 		if ((iasubopt->state <= 0) || (iasubopt->state > FTS_LAST)) {
-			log_fatal("Unknown iasubopt state %d at %s:%d", 
+			log_fatal("Unknown iasubopt state %d at %s:%d",
 				  iasubopt->state, MDL);
 		}
 		binding_state = binding_state_names[iasubopt->state-1];
-		if (fprintf(db_file, "    binding state %s;\n", 
+		if (fprintf(db_file, "    binding state %s;\n",
 			    binding_state) < 0) {
 			goto error_exit;
 		}
@@ -639,7 +660,7 @@ write_ia(const struct ia_xx *ia) {
 			if (write_binding_scope(db_file, bnd,
 						"\n    ") != ISC_R_SUCCESS)
 				goto error_exit;
-				
+
 		}
 
 		if (iasubopt->on_star.on_expiry) {
@@ -650,7 +671,7 @@ write_ia(const struct ia_xx *ia) {
 				goto error_exit;
 			write_statements(db_file,
 					 iasubopt->on_star.on_expiry, 6);
-			if (fprintf(db_file, "\n    }") < 0) 
+			if (fprintf(db_file, "\n    }") < 0)
 				goto error_exit;
 		}
 
@@ -697,12 +718,12 @@ write_server_duid(void) {
 		return 1;
 	}
 
-	/* 
-	 * If the lease file is corrupt, don't try to write any more 
-	 * leases until we've written a good lease file. 
+	/*
+	 * If the lease file is corrupt, don't try to write any more
+	 * leases until we've written a good lease file.
 	 */
 	if (lease_file_is_corrupt) {
-		if (!new_lease_file()) {
+		if (!new_lease_file(0)) {
 			return 0;
 		}
 	}
@@ -748,7 +769,7 @@ int write_failover_state (dhcp_failover_state_t *state)
 	const char *tval;
 
 	if (lease_file_is_corrupt)
-		if (!new_lease_file ())
+		if (!new_lease_file (0))
 			return 0;
 
 	errno = 0;
@@ -882,7 +903,7 @@ write_named_billing_class(const void *key, unsigned len, void *object)
 			if (fprintf(db_file, "  dynamic;\n") <= 0)
 				return ISC_R_IOERROR;
 		}
-	
+
 		if (class->lease_limit > 0) {
 			if (fprintf(db_file, "  lease limit %d;\n",
 				    class->lease_limit) <= 0)
@@ -893,7 +914,7 @@ write_named_billing_class(const void *key, unsigned len, void *object)
 			if (fprintf(db_file, "  match if ") <= 0)
 				return ISC_R_IOERROR;
 
-                        errno = 0;                                       
+                        errno = 0;
 			write_expression(db_file, class->expr, 5, 5, 0);
                         if (errno)
                                 return ISC_R_IOERROR;
@@ -919,7 +940,7 @@ write_named_billing_class(const void *key, unsigned len, void *object)
 			if (fprintf(db_file, ";\n") <= 0)
 				return ISC_R_IOERROR;
 		}
-	
+
 		if (class->statements != 0) {
                         errno = 0;
 			write_statements(db_file, class->statements, 8);
@@ -973,7 +994,7 @@ int write_billing_class (class)
 	int errors = 0;
 
 	if (lease_file_is_corrupt)
-		if (!new_lease_file ())
+		if (!new_lease_file (0))
 			return 0;
 
 	if (!class -> superclass) {
@@ -1028,7 +1049,7 @@ int commit_leases ()
 	if (count && cur_time - write_time > LEASE_REWRITE_PERIOD) {
 		count = 0;
 		write_time = cur_time;
-		new_lease_file();
+		new_lease_file(0);
 	}
 	return (1);
 }
@@ -1048,9 +1069,9 @@ int commit_leases_timed()
 	return (1);
 }
 
-void db_startup (testp)
-	int testp;
+void db_startup (int test_mode)
 {
+	const char *current_db_path;
 	isc_result_t status;
 
 #if defined (TRACING)
@@ -1077,22 +1098,26 @@ void db_startup (testp)
 	   append it, so we create one immediately (maybe this isn't
 	   the best solution... */
 	if (trace_playback ()) {
-		new_lease_file ();
+		new_lease_file (0);
 	}
 #endif
-	if (!testp) {
-		db_file = fopen (path_dhcpd_db, "a");
-		if (!db_file)
-			log_fatal ("Can't open %s for append.", path_dhcpd_db);
-		expire_all_pools ();
+	/* expire_all_pools will cause writes to the "current" lease file.
+	* Therefore, in test mode we need to point db_file to a disposable
+	* file to protect the original lease file. */
+	current_db_path = (test_mode ? "/dev/null" : path_dhcpd_db);
+	db_file = fopen (current_db_path, "a");
+	if (!db_file) {
+		log_fatal ("Can't open %s for append.", current_db_path);
+	}
+
+	expire_all_pools ();
 #if defined (TRACING)
-		if (trace_playback ())
-			write_time = cur_time;
-		else
+	if (trace_playback ())
+		write_time = cur_time;
+	else
 #endif
-			time(&write_time);
-		new_lease_file ();
-	}
+		time(&write_time);
+	new_lease_file (test_mode);
 
 #if defined(REPORT_HASH_PERFORMANCE)
 	log_info("Host HW hash:   %s", host_hash_report(host_hw_addr_hash));
@@ -1105,7 +1130,7 @@ void db_startup (testp)
 #endif
 }
 
-int new_lease_file ()
+int new_lease_file (int test_mode)
 {
 	char newfname [512];
 	char backfname [512];
@@ -1194,6 +1219,14 @@ int new_lease_file ()
 	if (!write_leases ())
 		goto fail;
 
+	if (test_mode) {
+		log_debug("Lease file test successful,"
+			  " removing temp lease file: %s",
+			  newfname);
+		(void)unlink (newfname);
+		return (1);
+	}
+
 #if defined (TRACING)
 	if (!trace_playback ()) {
 #endif
@@ -1226,7 +1259,7 @@ int new_lease_file ()
 #if defined (TRACING)
 	}
 #endif
-	
+
 	/* Move in the new file... */
 	if (rename (newfname, path_dhcpd_db) < 0) {
 		log_error ("Can't install new lease database %s to %s: %m",
